@@ -182,11 +182,7 @@ debuggerLoggerAction h a b c d = do
 -- 'BreakpointStatus' being set.
 --
 -- Returns @True@ when the breakpoint status is changed.
-#if MIN_VERSION_ghc(9,14,2)
 registerBreakpoint :: GHC.BreakpointId -> BreakpointStatus -> BreakpointKind -> Debugger (Bool, [GHC.InternalBreakpointId])
-#else
-registerBreakpoint :: GHC.BreakpointId -> BreakpointStatus -> BreakpointKind -> Debugger Bool
-#endif
 registerBreakpoint bp@GHC.BreakpointId
                     { GHC.bi_tick_mod = mod
                     , GHC.bi_tick_index = bid } status kind = do
@@ -194,13 +190,9 @@ registerBreakpoint bp@GHC.BreakpointId
   -- Set breakpoint in GHC session
   let breakpoint_count = breakpointStatusInt status
   hsc_env <- GHC.getSession
-#if MIN_VERSION_ghc(9,14,2)
   internal_break_ids <- getInternalBreaksOf bp
   forM_ internal_break_ids $ \ibi -> do
     GHC.setupBreakpoint (hscInterp hsc_env) ibi breakpoint_count
-#else
-  GHC.setupBreakpoint (hscInterp hsc_env) bp breakpoint_count
-#endif
 
   -- Register breakpoint in Debugger state
   brksRef <- asks activeBreakpoints
@@ -240,21 +232,13 @@ registerBreakpoint bp@GHC.BreakpointId
 
   -- no races since the debugger execution is run in a single thread
   liftIO $ writeIORef brksRef newBrks
-#if MIN_VERSION_ghc(9,14,2)
   return (changed, internal_break_ids)
-#else
-  return changed
-#endif
 
 
 -- | Get a list with all currently active breakpoints on the given module (by path)
 --
 -- If the path argument is @Nothing@, get all active function breakpoints instead
-#if MIN_VERSION_ghc(9,14,2)
 getActiveBreakpoints :: Maybe FilePath -> Debugger [GHC.InternalBreakpointId]
-#else
-getActiveBreakpoints :: Maybe FilePath -> Debugger [GHC.BreakpointId]
-#endif
 getActiveBreakpoints mfile = do
   m <- asks activeBreakpoints >>= liftIO . readIORef
   case mfile of
@@ -262,11 +246,7 @@ getActiveBreakpoints mfile = do
       mms <- getModuleByPath file
       case mms of
         Right ms ->
-#if MIN_VERSION_ghc(9,14,2)
           concat <$> mapM getInternalBreaksOf
-#else
-          return
-#endif
             [ GHC.BreakpointId mod bix
             | (mod, im) <- moduleEnvToList m
             , mod == ms_mod ms
@@ -277,11 +257,7 @@ getActiveBreakpoints mfile = do
           displayWarnings [e]
           return []
     Nothing -> do
-#if MIN_VERSION_ghc(9,14,2)
       concat <$> mapM getInternalBreaksOf
-#else
-      return
-#endif
         [ GHC.BreakpointId mod bix
         | (mod, im) <- moduleEnvToList m
         , (bix, (status, kind)) <- IM.assocs im
@@ -452,7 +428,6 @@ instance GHC.GhcMonad Debugger where
 
 --------------------------------------------------------------------------------
 
-#if MIN_VERSION_ghc(9,14,2)
 -- | Find all the internal breakpoints that use the given source-level breakpoint id
 getInternalBreaksOf :: BreakpointId -> Debugger [InternalBreakpointId]
 getInternalBreaksOf bi = do
@@ -460,4 +435,3 @@ getInternalBreaksOf bi = do
   return $
     fromMaybe [] {- still not found after refresh -} $
       lookupBreakpointOccurrences bs bi
-#endif
