@@ -4,17 +4,23 @@
 {-# LANGUAGE LambdaCase #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RecordWildCards #-}
+{-# LANGUAGE ViewPatterns #-}
 
 -- | TODO: This module should be called Launch.
 module Development.Debug.Adapter.Init where
 
+import GHC.IO.Handle
+import System.Process
+import qualified Data.ByteString as BS
 import qualified Data.Text as T
 import qualified Data.Text.IO as T
+import qualified Data.Text.Encoding as T
 import qualified System.Process as P
+import Control.Monad.Except
+import Control.Monad.Trans
 import Data.Function
 import Data.Functor
-import Data.Version (Version(..), showVersion, makeVersion)
-import Control.Monad.IO.Class
+import Data.Maybe
 import System.IO
 import GHC.IO.Encoding
 import Control.Monad.Catch
@@ -28,7 +34,6 @@ import System.FilePath
 
 import Development.Debug.Adapter
 import Development.Debug.Adapter.Exit
-import Development.Debug.Adapter.Flags
 import GHC.Debugger.Logger
 import qualified Development.Debug.Adapter.Output as Output
 
@@ -39,6 +44,7 @@ import GHC.Debugger.Interface.Messages hiding (Command, Response)
 
 import DAP
 import Development.Debug.Adapter.Handles
+import Development.Debug.Session.Setup
 
 --------------------------------------------------------------------------------
 -- * Logging
@@ -62,16 +68,16 @@ data LaunchArgs
   = LaunchArgs
   { __sessionId :: Maybe String
     -- ^ SessionID, set by VSCode client
-  , projectRoot :: FilePath
+  , projectRoot :: Maybe FilePath
     -- ^ Absolute path to the project root
-  , entryFile :: FilePath
+  , entryFile :: Maybe FilePath
     -- ^ The file with the entry point e.g. @app/Main.hs@
-  , entryPoint :: String
+  , entryPoint :: Maybe String
     -- ^ Either @main@ or a function name
-  , entryArgs :: [String]
+  , entryArgs :: Maybe [String]
     -- ^ The arguments to either set as environment arguments when @entryPoint = "main"@
     -- or function arguments otherwise.
-  , extraGhcArgs :: [String]
+  , extraGhcArgs :: Maybe [String]
     -- ^ Additional arguments to pass to the GHC invocation inferred by hie-bios for this project
   } deriving stock (Show, Eq, Generic)
     deriving anyclass FromJSON
@@ -80,41 +86,38 @@ data LaunchArgs
 -- * Launch Debugger
 --------------------------------------------------------------------------------
 
+-- | Exception type for when hie-bios initialization fails
+newtype InitFailed = InitFailed String deriving Show
+
 -- | Initialize debugger
 --
--- Returns @True@ if successful.
-initDebugger :: Recorder (WithSeverity InitLog) -> LaunchArgs -> DebugAdaptor Bool
-initDebugger l LaunchArgs{__sessionId, projectRoot, entryFile = entryFile, entryPoint, entryArgs, extraGhcArgs} = do
+-- Returns @()@ if successful, throws @InitFailed@ otherwise
+initDebugger :: Recorder (WithSeverity InitLog) -> Bool -> LaunchArgs -> ExceptT InitFailed DebugAdaptor ()
+initDebugger l supportsRunInTerminal
+               LaunchArgs{ __sessionId
+                         , projectRoot = givenRoot
+                         , entryFile = entryFileMaybe
+                         , entryPoint = fromMaybe "main" -> entryPoint
+                         , entryArgs  = fromMaybe [] -> entryArgs
+                         , extraGhcArgs = fromMaybe [] -> extraGhcArgs
+                         } = do
   syncRequests  <- liftIO newEmptyMVar
   syncResponses <- liftIO newEmptyMVar
+  syncProxyIn   <- liftIO newChan
+  syncProxyOut  <- liftIO newChan
+  syncProxyErr  <- liftIO newChan
+
+  entryFile <- case entryFileMaybe of
+    Nothing -> throwError $ InitFailed "Missing \"entryFile\" key in debugger configuration"
+    Just ef -> pure ef
+
+  projectRoot <- maybe (liftIO getCurrentDirectory) pure givenRoot
+
   let hieBiosLogger = cmapWithSev FlagsLog l
-  cradle <- liftIO (hieBiosCradle hieBiosLogger projectRoot entryFile) >>=
-    \ case
-      Left e ->
-        exitWithMsg e
-
-      Right c -> pure c
-
-  Output.console $ T.pack "Checking GHC version against debugger version..."
-  -- GHC is found in PATH (by hie-bios as well).
-  actualVersion <- liftIO (hieBiosRuntimeGhcVersion hieBiosLogger cradle) >>=
-    \ case
-      Left e ->
-        exitWithMsg e
-      Right c -> pure c
-  -- Compare the GLASGOW_HASKELL version (e.g. 913) with the actualVersion (e.g. 9.13.1):
-  when (compileTimeGhcWithoutPatchVersion /= forgetPatchVersion actualVersion) $ do
-    exitWithMsg $ "Aborting...! The GHC version must be the same which " ++
-                    "ghc-debug-adapter was compiled against (" ++
-                      showVersion compileTimeGhcWithoutPatchVersion++
-                        "). Instead, got " ++ (showVersion actualVersion) ++ "."
-
-  Output.console $ T.pack "Discovering session flags with hie-bios..."
-  mflags <- liftIO (hieBiosFlags hieBiosLogger cradle projectRoot entryFile)
-  case mflags of
-    Left e -> exitWithMsg e
-    Right flags -> do
-
+  liftIO (runExceptT (hieBiosSetup hieBiosLogger projectRoot entryFile)) >>= \case
+    Left e              -> throwError $ InitFailed e
+    Right (Left e)      -> lift       $ exitWithMsg e
+    Right (Right flags) -> do
       let nextFreshBreakpointId = 0
           breakpointMap = mempty
           defaultRunConf = Debugger.RunDebuggerSettings
@@ -138,43 +141,69 @@ initDebugger l LaunchArgs{__sessionId, projectRoot, entryFile = entryFile, entry
       finished_init <- liftIO $ newEmptyMVar
 
       let absEntryFile = normalise $ projectRoot </> entryFile
-      registerNewDebugSession (maybe "debug-session" T.pack __sessionId) DAS{entryFile=absEntryFile,..}
-        [ debuggerThread l finished_init writeDebuggerOutput projectRoot flags extraGhcArgs absEntryFile defaultRunConf syncRequests syncResponses
+      lift $ registerNewDebugSession (maybe "debug-session" T.pack __sessionId) DAS{entryFile=absEntryFile,..}
+        [ debuggerThread l finished_init writeDebuggerOutput projectRoot flags
+            extraGhcArgs absEntryFile defaultRunConf syncRequests syncResponses
         , handleDebuggerOutput readDebuggerOutput
-        , stdoutCaptureThread
-        , stderrCaptureThread
+        , stdinForwardThread  supportsRunInTerminal syncProxyIn
+        , stdoutCaptureThread supportsRunInTerminal syncProxyOut
+        , stderrCaptureThread supportsRunInTerminal syncProxyErr
         ]
 
       -- Do not return until the initialization is finished
       liftIO (takeMVar finished_init) >>= \case
-        Right () -> return True
+        Right () -> pure ()
         Left e   -> do
           -- The process terminates cleanly with an error code (probably exit failure = 1)
           -- This can happen if compilation fails and the compiler exits cleanly.
           --
           -- Instead of signalInitialized, respond with error and exit.
-          exitCleanupWithMsg readDebuggerOutput e
-          return False
+          lift $ exitCleanupWithMsg readDebuggerOutput e
 
--- | This thread captures stdout from the debugger and sends it to the client.
+-- | This thread captures stdout from the debuggee and sends it to the client.
 -- NOTE, redirecting the stdout handle is a process-global operation. So this thread
--- will capture ANY stdout the debugger emits. Therefore you should never directly
+-- will capture ANY stdout the debuggee emits. Therefore you should never directly
 -- write to stdout, but always write to the appropiate handle.
-stdoutCaptureThread :: (DebugAdaptorCont () -> IO ()) -> IO ()
-stdoutCaptureThread withAdaptor = do
+stdoutCaptureThread :: Bool -> Chan BS.ByteString -> (DebugAdaptorCont () -> IO ()) -> IO ()
+stdoutCaptureThread runInTerminal syncOut withAdaptor = do
   withInterceptedStdout $ \_ interceptedStdout -> do
     forever $ do
       line <- liftIO $ T.hGetLine interceptedStdout
+      when runInTerminal $
+        writeChan syncOut $ T.encodeUtf8 (line <> T.pack "\n")
+
+      -- Always output to Debug Console
       withAdaptor $ Output.stdout line
 
 -- | Like 'stdoutCaptureThread' but for stderr
-stderrCaptureThread :: (DebugAdaptorCont () -> IO ()) -> IO ()
-stderrCaptureThread withAdaptor = do
+stderrCaptureThread :: Bool -> Chan BS.ByteString -> (DebugAdaptorCont () -> IO ()) -> IO ()
+stderrCaptureThread runInTerminal syncErr withAdaptor = do
   withInterceptedStderr $ \_ interceptedStderr -> do
     forever $ do
       line <- liftIO $ T.hGetLine interceptedStderr
+      when runInTerminal $
+        writeChan syncErr $ T.encodeUtf8 (line <> "\n")
+
+      -- Always output to Debug Console
       withAdaptor $ Output.stderr line
 
+stdinForwardThread :: Bool -> Chan BS.ByteString -> (DebugAdaptorCont () -> IO ()) -> IO ()
+stdinForwardThread runInTerminal syncIn _withAdaptor = do
+  when runInTerminal $ do
+    -- We need to hijack stdin to write to it
+
+    -- 1. Create a new pipe from writeEnd->readEnd
+    (readEnd, writeEnd) <- createPipe
+
+    -- 2. Substitute the read-end of the pipe by stdin
+    _ <- hDuplicateTo readEnd stdin
+    hClose readEnd -- we'll never need to read from readEnd
+
+    forever $ do
+      i <- readChan syncIn
+      -- 3. Write to write-end of the pipe
+      BS.hPut writeEnd i >> hFlush writeEnd
+
 -- | The main debugger thread launches a GHC.Debugger session.
 --
 -- Then, forever:
@@ -245,7 +274,7 @@ debuggerThread recorder finished_init writeDebuggerOutput workDir HieBiosFlags{.
       hPutStrLn stderr m
       putMVar replies (Aborted m)
 
--- | Reads from the read end of the handle to which the GHC debugger writes compiler messages.
+-- | Reads from the read end of the handle to which the debugger writes compiler messages.
 -- Writes the compiler messages to the client console
 handleDebuggerOutput :: Handle
                      -> (DebugAdaptorCont () -> IO ())
@@ -261,15 +290,3 @@ handleDebuggerOutput readDebuggerOutput withAdaptor = do
           -- Cleanly exit when readDebuggerOutput is closed or thread is killed.
           return ()
 
-compileTimeGhcWithoutPatchVersion :: Version
-compileTimeGhcWithoutPatchVersion =
-  let
-    versionNumber = __GLASGOW_HASKELL__ :: Int
-    (major, minor) = divMod versionNumber 100
-  in
-    makeVersion [major, minor]
-
-forgetPatchVersion :: Version -> Version
-forgetPatchVersion v = case versionBranch v of
-  (major:minor:_patches) -> makeVersion [major, minor]
-  _ -> v
